package com.chinacreator.video.restful.dao;

import com.chinacreator.video.restful.bean.TlVerifiCationCodeBean;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Component;


@Component
public interface SendSmsCodeDao {

    String checkValidatTime(@Param("cpid") String cpid, @Param("phone") String phone);

    void insertTlVerifiCationCode(TlVerifiCationCodeBean tvcc);

    String checkTransfer(@Param("userid") String userid);

    String checkArea(@Param("phone") String phone);
}
